#include <EEPROM.h>
#include <LocoNet.h>

/******************************************************************************
 *
 *  Copyright (C) 2014 Daniel Guisado - http://www.clubncaldes.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************

 LIBRARIES:
 This software needs LocoNet libraries from MRRWA:
 http://sourceforge.net/projects/mrrwa/
 
 SHIELDS:
 This software uses  Loconet Shield GCA185 from Peter Giling and MP3 Shield from Elechouse
 http://www.elechouse.com/elechouse/index.php?main_page=product_info&products_id=2193
 http://wiki.rocrail.net/doku.php?id=gca185-en
 
 RESERVED PINS:
 pins 7,8 for Loconet Shield
 pins 0,1 for MP3 Shield
 
 DESCRIPTION:
 This is a Loconet Outputs and Sound module configured through Uhlenbrock LNCVs:

        LNCV	MEANING	                                DEFAULT
        0	Module Address	                          1
        1	Address pin 0 (0=Sound, 99=Debug)	  0
        2	Address pin 1 (0=Sound, 99=Debug)	  0
        3	Address pin 2	                          1
        4	Address pin 3	                          2
        5	Address pin 4	                          3
        6	Address pin 5	                          4
        7	Address pin 6	                          5
        8	Address pin 7 (0=Reserved Loconet)	  0
        9	Address pin 8 (0=Reserved Loconet)	  0
        10	Address pin 9	                          6
        11	Address pin 10	                          7
        12	Address pin 11	                          8
        13	Address pin 12	                          9
        14	Address pin 13	                          10
        15	Address pin 14 A0	                  11
        16	Address pin 15 A1	                  12
        17	Address pin 16 A2	                  13
        18	Address pin 17 A3	                  14
        19	Address pin 18 A4	                  15
        20	Address pin 19 A5	                  16
        21	Address first sound (0=No sounds)	  50
        22	Total number of sounds (0=No sounds)	  10
        23	MP3 Volume (0 to 30)	                  15
        24	Sound Source (0=SD card, 1=SPI, 2=Udisk)  2
        50	Config command:	
        	1 - Store config in EEPROM	
        	2- Reset Module	

 ******************************************************************************/

#define LOCONET_TX_PIN 7
#define LNCV_COUNT 25
#define ARTNR 9001

#define MP3_SDCARD 0xA0
#define MP3_SPI    0xA1
#define MP3_UDISK  0xA2

uint16_t lncv[LNCV_COUNT];
lnMsg *LnPacket;
LocoNetCVClass lnCV;

boolean programmingMode=false;
boolean debugMode=false;

byte MP3_source;  //Source of sounds SDCARD | SPI | UDISK
byte MP3_volume;  //Volume
unsigned char cmd_buf[10];

void setup() 
{
  int i=0;
  
  loadSettings();
  if (lncv[1]!=0 && lncv[1]!=99) lncv[1]=99;
  if (lncv[2]!=0 && lncv[2]!=99) lncv[1]=99;  
  lncv[8]=0; lncv[9]=0; //Reservado Loconet
  debugMode=(lncv[1]==99 && lncv[2]==99);
  
  LocoNet.init(LOCONET_TX_PIN);
  
  Serial.begin(9600);
  Serial.print("Module Address is now: ");
  Serial.print(lncv[0]);
  Serial.print("\n");
  
  if (debugMode)
    Serial.print("Starting module 90010 test\n");

  programmingMode = false;

  pinMode(2,OUTPUT);
  digitalWrite(2,HIGH);
  pinMode(3,OUTPUT);
  digitalWrite(3,HIGH);
  pinMode(4,OUTPUT);
  digitalWrite(4,HIGH);
  pinMode(5,OUTPUT);
  digitalWrite(5,HIGH);
  pinMode(6,OUTPUT);
  digitalWrite(6,HIGH);
  pinMode(9,OUTPUT);
  digitalWrite(9,HIGH);
  pinMode(10,OUTPUT);
  digitalWrite(10,HIGH);
  pinMode(11,OUTPUT);
  digitalWrite(11,HIGH);
  pinMode(12,OUTPUT);
  digitalWrite(12,HIGH);
  pinMode(13,OUTPUT);
  digitalWrite(13,HIGH);

  pinMode(14,OUTPUT);
  digitalWrite(14,HIGH);
  pinMode(15,OUTPUT);
  digitalWrite(15,HIGH);
  pinMode(16,OUTPUT);
  digitalWrite(16,HIGH);
  pinMode(17,OUTPUT);
  digitalWrite(17,HIGH);
  pinMode(18,OUTPUT);
  digitalWrite(18,HIGH);
  pinMode(19,OUTPUT);
  digitalWrite(19,HIGH);

  //MP3 volume init
  if (!debugMode)
  {
    delay(1000);
    MP3_volume=lowByte(lncv[23]);
    if (MP3_volume>30) MP3_volume=15;
    MP3setVolume();
    
    // set play mode sing
    cmd_buf[0] = 0x7E;          // START
    cmd_buf[1] = 0x03;          // Length
    cmd_buf[2] = 0xA9;          // Command SET MODE
    cmd_buf[3] = 0x00;          // set mode
    cmd_buf[4] = 0x7E;          // END
    ArduinoMP3Shield_SendCMD(cmd_buf, 5);  
    delay(1000);
    //Plays first file in internal memory
    MP3_source=MP3_SPI;
    MP3playFile(1);
    delay(1000);
    //Sets MP3 source
    if (lncv[24]<3 && lncv[24]>=0)
      MP3_source=0xA0+lncv[24];
    else
      MP3_source=MP3_UDISK;
  }
}

void loop() 
{
  LnPacket = LocoNet.receive();
  if (LnPacket) 
  {
    uint8_t packetConsumed(LocoNet.processSwitchSensorMessage(LnPacket));
    if (packetConsumed == 0) 
    {
      packetConsumed = lnCV.processLNCVMessage(LnPacket);
    }
  }
}

/*************************************************************************/
/*          LNCV LOCONET PROGRAMMING FUNCTIONS                           */
/*************************************************************************/
void commitLNCVUpdate() {
  if (debugMode)
  {
	Serial.print("Module Address is now: ");
	Serial.print(lncv[0]);
	Serial.print("\n");
  }
}

void dumpPacket(UhlenbrockMsg & ub) {
  if (debugMode)
  {
	Serial.print(" PKT: ");
	Serial.print(ub.command, HEX);
	Serial.print(" ");
	Serial.print(ub.mesg_size, HEX);
	Serial.print(" ");
	Serial.print(ub.SRC, HEX);
	Serial.print(" ");
	Serial.print(ub.DSTL, HEX);
	Serial.print(" ");
	Serial.print(ub.DSTH, HEX);
	Serial.print(" ");
	Serial.print(ub.ReqId, HEX);
	Serial.print(" ");
	Serial.print(ub.PXCT1, HEX);
	Serial.print(" ");
	for (int i(0); i < 8; ++i) {
		Serial.print(ub.payload.D[i], HEX);
		Serial.print(" ");
	}
	Serial.write("\n");
  }
}

int8_t notifyLNCVread(uint16_t ArtNr, uint16_t lncvAddress, uint16_t, uint16_t & lncvValue) 
{
  if (debugMode)
  {
    Serial.print("READ LNCV (");
    Serial.print(lncvAddress);
    Serial.print("): ");
  }
  
  // Step 1: Can this be addressed to me?
  // All ReadRequests contain the ARTNR. For starting programming, we do not accept the broadcast address.
  if (programmingMode) 
  {
    if (ArtNr == ARTNR) 
    {
      if (lncvAddress < LNCV_COUNT) 
      {
	lncvValue = lncv[lncvAddress];
        if (debugMode) 
        {
          Serial.print(" LNCV Value: ");
	  Serial.print(lncvValue);
	  Serial.print("\n");
        }
	return LNCV_LACK_OK;
      } 
      else 
      {
	// Invalid LNCV address, request a NAXK
        if (debugMode) Serial.print(" Invalid LNCV addres\n");
	return LNCV_LACK_ERROR_UNSUPPORTED;
      }
    } 
    else 
    {
      if (debugMode) Serial.print("ArtNr invalid.\n");
      return -1;
    }
  }
  else
  {
    if (debugMode) Serial.print("Ignoring Request.\n");
    return -1;
  }
}

int8_t notifyLNCVprogrammingStart(uint16_t & ArtNr, uint16_t & ModuleAddress) 
{
  // Enter programming mode. If we already are in programming mode,
  // we simply send a response and nothing else happens.
  if (debugMode) Serial.print("LNCV PROGRAMMING START: ");
  if (ArtNr == ARTNR) 
  {
    if (debugMode) Serial.print("Artnr OK, ");
    if (ModuleAddress == lncv[0]) 
    {
      if (debugMode) Serial.print("Module Addr. OK \n");
      programmingMode = true;
      return LNCV_LACK_OK;
    } 
    else if (ModuleAddress == 0xFFFF) 
    {
      if (debugMode) Serial.print("Broadcast Addr.\n");
      ModuleAddress = lncv[0];
      return LNCV_LACK_OK;
    }
  }
  if (debugMode) Serial.print("Ignoring Request.\n");
  return -1;
}

int8_t notifyLNCVwrite(uint16_t ArtNr, uint16_t lncvAddress,uint16_t lncvValue) 
{
  if (debugMode) 
  {
    Serial.print("LNCV WRITE (LNCV ");
    Serial.print(lncvAddress); Serial.print(" = "); Serial.print(lncvValue);
    Serial.print("): ");
  }
  //  dumpPacket(ub);
  if (!programmingMode) 
  {
    if (debugMode) Serial.print("not in Programming Mode.\n");
    return -1;
  }

  if (ArtNr == ARTNR) 
  {
    if (debugMode) Serial.print("Artnr OK, ");
    
    //Special LNCV 50 to command a Save (1) or Reset (99), it's not really a LNCV
    if (lncvAddress==50)
    {
      if (lncvValue==1)
      {
        if (debugMode) Serial.print("Save request\n");
        saveSettings();
        return LNCV_LACK_OK;
      }
      if (lncvValue=99)
      {
        if (debugMode) Serial.print("Reset request\n");
        lncv[0]=1; lncv[1]=0; lncv[2]=0;
        lncv[3]=1; lncv[4]=2; lncv[5]=3; lncv[6]=4; lncv[7]=5; 
        lncv[8]=0; lncv[9]=0;
        lncv[10]=6; lncv[11]=7; lncv[12]=8; lncv[13]=9; lncv[14]=10;
        lncv[15]=11; lncv[16]=12; lncv[17]=13; lncv[18]=14; lncv[19]=15;
        lncv[20]=16; lncv[21]=50; lncv[22]=10; lncv[23]=15; lncv[24]=2;
        saveSettings();
        return LNCV_LACK_OK;
      }
      if (debugMode) Serial.print("LNCV 50 value unsupported\n");
      return LNCV_LACK_ERROR_UNSUPPORTED;
    }
    
    //Valid LNCV number
    if (lncvAddress < LNCV_COUNT) 
    {
      if (debugMode) Serial.println("OK\n");
      lncv[lncvAddress] = lncvValue;
      return LNCV_LACK_OK;
    }
    else 
    {
      if (debugMode) Serial.println("Value unsupported\n");
      return LNCV_LACK_ERROR_UNSUPPORTED;
    }
  }
  else 
  {
    if (debugMode) Serial.print("Artnr Invalid.\n");
    return -1;
  }
}

void notifyLNCVprogrammingStop(uint16_t ArtNr, uint16_t ModuleAddress) 
{
  if (debugMode) Serial.print("LNCV Programming STOP ");
  if (programmingMode) 
  {
    if (ArtNr == ARTNR && ModuleAddress == lncv[0]) 
    {
      programmingMode = false;
      if (debugMode) Serial.print("End Programing Mode.\n");
      commitLNCVUpdate();
    }
    else 
    {
      if (ArtNr != ARTNR) 
      {
        if (debugMode) Serial.print("Wrong Artnr.\n");
	return;
      }
      if (ModuleAddress != lncv[0]) 
      {
        if (debugMode) Serial.print("Wrong Module Address.\n");
	return;
      }
    }
  }
  else 
  {
    if (debugMode) Serial.print("Ignoring Request.\n");
  }
}

/*************************************************************************/
/*          LOCONET SWITCH AND SENSOR FUNCTIONS                          */
/*************************************************************************/
void notifySensor( uint16_t Address, uint8_t State )
{
  if (debugMode) 
  {
    Serial.print("LOCONET Sensor: ");
    Serial.print(Address, DEC);
    Serial.print(" - ");
    Serial.println( State ? "Active" : "Inactive" );
  }
}

void notifySwitchRequest( uint16_t Address, uint8_t Output, uint8_t Direction )
{
  int i=0;
  
  if (debugMode) 
  {
    Serial.print("LOCONET Switch Request: ");
    Serial.print(Address, DEC);
    Serial.print(':');
    Serial.print(Direction ? "Closed" : "Thrown");
    Serial.print(" - ");
    Serial.println(Output ? "On" : "Off");
  }
  
  if (!Output) return;
  
  for (i==1; i < 21; i++) 
  {
    if (lncv[i] == Address)
    {
      if (debugMode) Serial.print("CAMBIANDO SALIDA "); Serial.println(i-1, DEC);
      Direction ? digitalWrite(i-1, HIGH) : digitalWrite(i-1, LOW);
      return;
    }
  }
  
  if (Address>=lncv[21] && Address<lncv[21]+lncv[22] && Direction)
  {
    if (debugMode) Serial.print("REPRODUCIENDO SONIDO "); Serial.println(Address-lncv[21]+1);
    MP3playFile(Address-lncv[21]+1);
  }
  
}

void notifySwitchReport( uint16_t Address, uint8_t Output, uint8_t Direction )
{
  if (debugMode) 
  {
    Serial.print("LOCONET Switch Report: ");
    Serial.print(Address, DEC);
    Serial.print(':');
    Serial.print(Direction ? "Closed" : "Thrown");
    Serial.print(" - ");
    Serial.println(Output ? "On" : "Off");
  }
}

void notifySwitchState( uint16_t Address, uint8_t Output, uint8_t Direction )
{
  if (debugMode) 
  {
    Serial.print("LOCONET Switch State: ");
    Serial.print(Address, DEC);
    Serial.print(':');
    Serial.print(Direction ? "Closed" : "Thrown");
    Serial.print(" - ");
    Serial.println(Output ? "On" : "Off");
  }
}


/*************************************************************************/
/*          SOUND FUNCTIONS (SAINSMART MP3)                              */
/*************************************************************************/
void ArduinoMP3Shield_SendCMD(unsigned char *cmd_buf, unsigned len)
{
    unsigned i;
    
    if (debugMode) return;
    
    for(i=0; i<len; i++){
        Serial.write(cmd_buf[i]);
    }
}

void MP3setVolume()
{
  /** set volume */
  cmd_buf[0] = 0x7E;          // START
  cmd_buf[1] = 0x03;          // Length
  cmd_buf[2] = 0xA7;          // Command
  cmd_buf[3] = MP3_volume;    // new volume
  cmd_buf[4] = 0x7E;          // END
  ArduinoMP3Shield_SendCMD(cmd_buf, 5);
}

void MP3playFile(uint8_t pFileNum)
{
  cmd_buf[0] = 0x7E;          // START
  cmd_buf[1] = 0x04;          // Length
  cmd_buf[2] = MP3_source;    // Command
  cmd_buf[3] = 0x00;          // file number high byte
  cmd_buf[4] = pFileNum;    // file number low byte
  cmd_buf[5] = 0x7E;          // END
  ArduinoMP3Shield_SendCMD(cmd_buf, 6); 
} //end playFile


/*************************************************************************/
/*          EEPROM FUNCTIONS                                             */
/*************************************************************************/

void saveSettings()
{
  int i=0;
  
  for (i=0; i < LNCV_COUNT; i++) 
  {
    EEPROM.write(i*2,highByte(lncv[i]));
    EEPROM.write(i*2+1,lowByte(lncv[i]));
  }
}

void loadSettings()
{
  int i=0;
  
  for (i=0; i < LNCV_COUNT; i++) 
  {
    lncv[i]=word(EEPROM.read(i*2),EEPROM.read(i*2+1));
  }
}

