#include <LocoNet.h>

// LocoNet Packet Monitor
// Demonstrates the use of the:
//
//   LocoNet.processSwitchSensorMessage(LnPacket)  


static  lnMsg        *LnPacket;
static  LnBuf        LnTxBuffer;

const int dAnz = 18;
int dPin[dAnz]       = {0,1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19};
int dAdr[dAnz]       = {1 ,2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18}; 

#define timefalling 0  //Verzögerung der Rückmeldung HIGH -> LOW
#define timerising 100    //Verzögerung der Rückmeldung LOW -> HIGH

int dtime[dAnz]  = {0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; 
boolean dstate[dAnz] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false};

int count = -1;    //Zähler für PIN Abfrage, beginne bei 0

void setup()
{
  // First initialize the LocoNet interface
  LocoNet.init(7);

  Serial.end();
  
  //Initialisierung der Eingänge
  for (int i = 0; i < dAnz; i++) {
    pinMode(dPin[i], INPUT);       //Pin auf Eingang
    digitalWrite(dPin[i], HIGH);   //Pullup aktivieren
    if (dstate[i] == digitalRead(dPin[i]))    //Vorbelegung setzten!
      dstate[i] = true;
  }
}

void loop()
{  
  // Check for any received LocoNet packets
  LnPacket = LocoNet.receive();
  
  if( LnPacket )
  {
      // First print out the packet in HEX
    Serial.print("RX: ");
    uint8_t msgLen = getLnMsgSize(LnPacket); 
    for (uint8_t x = 0; x < msgLen; x++)
    {
      uint8_t val = LnPacket->data[x];
        // Print a leading 0 if less than 16 to make 2 HEX digits
      if(val < 16)
        Serial.print('0');
        
      Serial.print(val, HEX);
      Serial.print(' ');
    }
    
      // If this packet was not a Switch or Sensor Message then print a new line 
    if(!LocoNet.processSwitchSensorMessage(LnPacket))
      Serial.println();
  }
    
  count++;    //nächsten Input Abfragen
  if (count >= dAnz)    //Max Input prüfen
    count = 0;        //beim Ersten beginnen
  if (dstate[count] != digitalRead(dPin[count])) {  //Eingang hat sich geändert?
    dtime[count]++;
    if ((dstate[count] == true) && (dtime[count] >= timefalling)) {    //falling (high -> low)
       sendSensor(dAdr[count], dstate[count]);     //aktuellen Status senden
       dstate[count] = !dstate[count];             //Speicher aktualisieren
       dtime[count] = 0;
    }
    else if ((dstate[count] == false) && (dtime[count] >= timerising)) {     //rising (low -> high)
       sendSensor(dAdr[count], dstate[count]);     //aktuellen Status senden
       dstate[count] = !dstate[count];             //Speicher aktualisieren
       dtime[count] = 0;     
    }
  }
}

// Rückmeldesonsor Daten senden
void sendSensor(int Adr, boolean state) {
  //Adressen von 1 bis 4096 akzeptieren
  if (Adr <= 0)        //nur korrekte Adressen
    return;
  Adr = Adr - 1;    
  int D2 = Adr >> 1;    //Adresse Teil 1 erstellen
  bitWrite(D2,7,0);        //A1 bis A7
    
  int D3 = Adr >> 8;     //Adresse Teil 2 erstellen, A8 bis A11
  bitWrite(D3,4, state);    //Zustand ausgeben
  bitWrite(D3,5,bitRead(Adr,0));    //Adr Bit0 = A0
    
  //Checksum bestimmen:  
  int D4 = 0xFF;        //Invertierung setzten
  D4 = OPC_INPUT_REP ^ D2 ^ D3 ^ D4;  //XOR
  bitWrite(D4,7,0);     //MSB Null setzten
  
  addByteLnBuf( &LnTxBuffer, OPC_INPUT_REP ) ;    //0xB2
  addByteLnBuf( &LnTxBuffer, D2 ) ;    //1. Daten Byte IN2
  addByteLnBuf( &LnTxBuffer, D3 ) ;    //2. Daten Byte IN2
  addByteLnBuf( &LnTxBuffer, D4 ) ;    //Prüfsumme
  addByteLnBuf( &LnTxBuffer, 0xFF ) ;    //Trennbit

        // Check to see if we have received a complete packet yet
  LnPacket = recvLnMsg( &LnTxBuffer );    //Senden vorbereiten
  if(LnPacket ) {        //korrektheit Prüfen
    LocoNet.send( LnPacket );  // Send the received packet from the PC to the LocoNet
  }  
}
